/*
 *  Copyright: Collin Mulliner <collin[at]mulliner.org>
 *  Web: http://www.mulliner.org/nfc/
 *  
 *  License: GPLv2
 *  
 */

import java.util.Date;

import javax.microedition.contactless.TargetType;

import com.nokia.nfc.nxp.mfstd.MFKey;

public class NFCTag
{
	String uid;
	String url;
	String mapping;
	TargetType[] targetTypes;
	String[] connNames;
	//NDEFMessage message;
	//NDEFRecord[] records;
	byte[] data;
	int tag_size;
	static final char[] hexitem =  {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
	Date dateTime;
	int state;
	int createdBy;
	DebugInterface dbgout;
	MFKey wkey;
	int w_is_AorB; // 1 = A 2 = B
	MFKey rkey;
	int r_is_AorB; // 1 = A 2 = B
	
	public NFCTag()
	{
		state = 0;
		createdBy = 0;
	}
	
	public String getCardType(DebugInterface d)
	{
		String card_type = "";
		NFCTag tag = this;
		
		if (tag.url != null && tag.url.indexOf("mf1k") >= 0) {
			card_type = "Mifare 1k";
		}
		else if (tag.url != null && tag.url.indexOf("mf4k") >= 0) {
			card_type = "Mifare 4k";
		}
		else {
			for (int i = 0; i < tag.connNames.length; i++) {
				//if (d != null) d.debug(" c>" + tag.connNames[i] + "< ", 0);
				if (tag.connNames[i].indexOf("DESFire", 11) >= 0) {
					card_type = "DESFire";
					break;
				}
			}
		}
		
		for (int i = 0; i < tag.targetTypes.length; i++) {
			//if (d != null) d.debug(" t>" + tag.targetTypes[i].toString() + "< ", 0);
			if (tag.targetTypes[i].equals(TargetType.ISO14443_CARD) || tag.targetTypes[i].toString().indexOf("14443") >= 0) {
				card_type += " (ISO14443 card)";
			}
			else if (tag.targetTypes[i].equals(TargetType.RFID_TAG) || tag.targetTypes[i].toString().indexOf("RFID") >= 0) {
				card_type += " (RFID tag)";
			}
		}
		
		//if (d != null) d.debug(" ct>" + card_type + "< ", 0);
		
		if (card_type.length() == 0)
			card_type = "unknown";
		
		return card_type;
	}
	
	private String[] getLines(String data)
	{
		String[] outp;
		int num = 0;
		for (int i = 0; i < data.length(); i++) {
			if (data.charAt(i) == '\n')
				num++;
		}
		//dbgout.append(new String("found " + String.valueOf(num) + " lines "));
		outp = new String[num];
		int cos = 0;
		StringBuffer cs = null;
		for (int i = 0; i < data.length(); i++) {
			if (cs == null) {
				cs = new StringBuffer("");
			}
			if (cs != null) {
				if (data.charAt(i) != '\n')
					cs.append(data.charAt(i));
				else {
					outp[cos] = cs.toString();
					cos++;
					cs = null;
				}
			}
		}
		return outp;
	}
	
	public NFCTag(String text, DebugInterface d)
	{
		dbgout = d;
		createdBy = 2;
		state = 0;
		
		String[] lines = getLines(text);
		for (int i = 0; i < lines.length; i++) {
			//dbgout.debug(new String(" " + lines[i]+ " "), 3);
			if (lines[i].startsWith("DATA:")) {
				//message = new NDEFMessage(fromHex(lines[i], 5), 0);
				//byte b[] = fromHex(lines[i], 5);
				
				// we only want raw data
				data = fromHex(lines[i], 5);
				break;
			}
		}
	}
	
	public String toReadTagCmd()
	{
		//byte data[] = message.toByteArray();
		int len = 6 + (data.length * 2); // data
		len += 5 + uid.length(); // uid
		StringBuffer ob = new StringBuffer(len + 6);
		//String tp = getCardType();
		
		// add padding
		if (len >= 10000) {}
		else if (len >= 1000)
			ob.append("0");
		else if (len >= 100)
			ob.append("00");
		else if (len >= 10)
			ob.append("000");
		else
			ob.append("0000");
		// command length
		ob.append(Integer.toString(len));
		ob.append("\n");
		
		// Tag UID
		ob.append("UID:");
		ob.append(uid);
		ob.append("\n");
		
		// Tag type
		//ob.append("Type:");
		//ob.append(tp);
		//ob.append("\n");
		
		// Tag RAW data
		ob.append("DATA:");
		//char b[] = toHex(data);
		//ob.append("\n");
			
		return ob.toString();
	}
	
	public byte[] getHex()
	{
		//return toHex(message.toByteArray());
		return toHex(data);
	}
	
	public byte[] toRMS()
	{
		//byte[] m = message.toByteArray();
		byte[] u = uid.getBytes();
		byte[] o = new byte[data.length + u.length + 1];
		
		o[0] = (byte) u.length;
		System.arraycopy(u, 0, o, 1, u.length);
		System.arraycopy(data, 0, o, 1+u.length, data.length);
		
		return o;
	}
	
	public void fromRMS(byte[] b)
	{
		int len = b[0];
		byte u[] = new byte[len];
		System.arraycopy(b, 1, u, 0, len);
		uid = new String(u);
		//message = new NDEFMessage(b, len+1);
		data = new byte[b.length - (len + 1)];
		System.arraycopy(b, len+1, data, 0, b.length - (len + 1));
	}
	
	public byte[] toHex(byte[] b)
	{
		byte out[] = new byte[b.length * 2];
		
		for (int i = 0; i < b.length; i++) {
			int val = b[i] & 0xFF;
			out[i*2] = (byte) hexitem[val/16];
			out[i*2+1] = (byte) hexitem[val%16];
		}
		
		return out;
	}
	
	public byte[] fromHex(String hex, int start)
	{
		int len = hex.length() - start;
		byte[] b = new byte[len/2];
		int bc = 0;
		
		//dbgout.debug(new String("len = " + String.valueOf(len) + " "), 3);
		
		for (int i = start; i < start + len; i+=2) {
			String cur = hex.substring(i, i+2);
			//dbgout.debug(cur, 3);
			b[bc] = (byte) Integer.parseInt(cur, 16);
			bc++;
		}
		
		return b;
	}
}